# Issues with input and output datatypes
GIA101 = 'Use `format="tabular,tsv"` for table data input parameters or a superset thereof.'
GIA102 = 'Use `tiff` instead of `tif`.'
GIA103 = 'Use `format="tabular"` for table data output.'

# Issues with the `command` or `configfiles` sections
GIA201 = 'Zarr input images require special treatment within the `command` of the tool.'
GIA202 = 'Cheetah code is invalid.'
GIA203 = 'Cheetah code for `configfile` recognized as JSON but JSON code is invalid.'
GIA204 = 'Cheetah code yields not the expected result.'
GIA205 = 'Command uses Python script that is not referenced under `required_files`.'
